"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const vue_1 = require("vue");
function useMergedState(defaultStateValue, option) {
    const { defaultValue, value = (0, vue_1.ref)() } = option || {};
    let initValue = typeof defaultStateValue === 'function' ? defaultStateValue() : defaultStateValue;
    if (value.value !== undefined) {
        initValue = (0, vue_1.unref)(value);
    }
    if (defaultValue !== undefined) {
        initValue = typeof defaultValue === 'function' ? defaultValue() : defaultValue;
    }
    const innerValue = (0, vue_1.ref)(initValue);
    const mergedValue = (0, vue_1.ref)(initValue);
    (0, vue_1.watchEffect)(() => {
        let val = value.value !== undefined ? value.value : innerValue.value;
        if (option === null || option === void 0 ? void 0 : option.postState) {
            val = option.postState(val);
        }
        mergedValue.value = val;
    });
    function triggerChange(newValue) {
        const preVal = mergedValue.value;
        innerValue.value = newValue;
        if ((0, vue_1.toRaw)(mergedValue.value) !== newValue && (option === null || option === void 0 ? void 0 : option.onChange)) {
            option.onChange(newValue, preVal);
        }
    }
    // Effect of reset value to `undefined`
    (0, vue_1.watch)(value, () => {
        innerValue.value = value.value;
    });
    return [mergedValue, triggerChange];
}
exports.default = useMergedState;
